#include <gtk/gtk.h>
#include <stdio.h>
#include "light.h"

#include "fifo.h"
#include "utils.h"

extern Config config;


static GList *fifo;		/* main fifo list */
static int fifo_index;		/* index */

static void fifo_add(const char *url, const char *note);
static void fifo_fill_clist(gpointer fucking_glib, gpointer dummy);
static void fifo_write_data(gpointer glib_sucks, gpointer dummy);
static void fifo_save(void);
static void light_add_fifo_cb(GtkEditable *entry, Browser *browser);

int fifo_size(void)
{
    return g_list_length(fifo);
}

void light_add_fifo(Browser *browser, const char *url, const char *title)
{
    GtkWidget *dialog;
    GtkWidget *vbox;
    GtkWidget *entry;

    dialog = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_window_set_title(GTK_WINDOW(dialog), "Fifo Add - Note");
    gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
    gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);

    vbox = gtk_vbox_new(FALSE, UI_SPACING);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), UI_SPACING);
    gtk_container_add(GTK_CONTAINER(dialog), vbox);

    entry = gtk_entry_new();
    /* if we had a link text, set that as default contents of the Fifo Add
     * window, and select it */
    if (title) {
	char *local = g_strdup(title);
	/* need to strstrip for <a href="">\nTitle Here\n</a> cases */
	gtk_entry_set_text(GTK_ENTRY(entry), g_strstrip(local));
	gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
	g_free(local);
    }
    gtk_box_pack_start(GTK_BOX(vbox), entry, TRUE, TRUE, 0);
    gtk_widget_grab_focus(entry);

    gtk_object_set_data(GTK_OBJECT(entry), "url", g_strdup(url));
    gtk_object_set_data(GTK_OBJECT(entry), "dialog", dialog);

    gtk_signal_connect(GTK_OBJECT(entry), "activate",
	    GTK_SIGNAL_FUNC(light_add_fifo_cb), browser);
    gtk_signal_connect(GTK_OBJECT(entry), "key_press_event",
	    GTK_SIGNAL_FUNC(dialog_esc_handler), dialog);

    gtk_widget_show_all(dialog);
}

void fifo_list(GtkWidget *clist)
{
    fifo_index = 1;
    g_list_foreach(fifo, fifo_fill_clist, clist);

}

void fifo_nuke(int index)
{
    MenuEntry *data;

    /* find the offending entry */
    data = g_list_nth_data(fifo, index);

    /* free the stuff */
    g_free(data->name);
    g_free(data->path);
    g_free(data);

    /* remove it */
    fifo = g_list_remove(fifo, data);

    /* save the fifo buffer */
    fifo_save();
}

void fifo_load(void)
{
    FILE *fp;
    char *path;
    MenuEntry *data;
    char name[512];
    char url[1024];
    
    path = g_strconcat(config.pref, "/fifo", NULL);
    fp = fopen(path, "r");
    g_free(path);
    
    /* empty file */
    if (!fp)
	return;

    while (fgets(name, 512, fp)) {
	if (fgets(url, 1024, fp)) {
	    data = g_new0(MenuEntry, 1);
	    g_strchomp(name);
	    g_strchomp(url);
	    data->name = g_strdup(name);
	    data->path = g_strdup(url);
	    fifo = g_list_append(fifo, data);
	}
    }
    fclose(fp);
}

static void fifo_save(void)
{
    char *path;
    FILE *fp = NULL;

    path = g_strconcat(config.pref, "/fifo", NULL);
    fp = fopen(path, "w");
    if (!fp)
	return;
    g_list_foreach(fifo, fifo_write_data, fp);
    fclose(fp);
    g_free(path);
}

void fifo_halt(void)
{
    g_list_free(fifo);
}

/* local functions */
static void fifo_add(const char *url, const char *note)
{
    MenuEntry *fifo_entry;
    GList *tmp;
    int length;

    fifo_entry = g_new0(MenuEntry, 1);

    fifo_entry->name = g_strdup(note);
    fifo_entry->path = g_strdup(url);

    /* add this at the top */
    fifo = g_list_prepend(fifo, fifo_entry);

    /* see if we need to delete some */
    length = g_list_length(fifo);

    /* too many elements, nuke some, but if FIFO_SIZE == 0
     * we have a unlimited length buffer */
    if ((FIFO_SIZE != 0) && (length > FIFO_SIZE)) {
	MenuEntry *data;

	tmp = g_list_last(fifo);
	fifo = g_list_remove_link(fifo, tmp);
	
	data = (MenuEntry *)tmp->data;
	g_free(data->name);
	g_free(data->path);
	g_free(data);

	g_list_free(tmp);
    }
    fifo_save();
}

static void fifo_write_data(gpointer glib_sucks, gpointer dummy)
{
    MenuEntry *data = (MenuEntry *)glib_sucks;
    FILE *fp = (FILE *)dummy;

    fprintf(fp, "%s\n%s\n", data->name, data->path);
}

static void fifo_fill_clist(gpointer fucking_glib, gpointer dummy)
{
    char *row[] = { NULL, NULL, NULL };

    MenuEntry *data = (MenuEntry *)fucking_glib;
    GtkWidget *clist = (GtkWidget *)dummy;

    row[0] = g_strdup_printf("%02d", fifo_index);
    row[1] = data->name;	/* these are already allocated */
    row[2] = data->path;

    gtk_clist_insert(GTK_CLIST(clist), fifo_index, row);
    g_free(row[0]);
    fifo_index++;
}

/* this happens when enter is pressed in the "add note" popup" */
static void light_add_fifo_cb(GtkEditable *entry, Browser *browser)
{
    GtkWidget *dialog;
    char *url;
    char *title;

    dialog = gtk_object_get_data(GTK_OBJECT(entry), "dialog");

    title = gtk_editable_get_chars(entry, 0, -1);
    url = gtk_object_get_data(GTK_OBJECT(entry), "url");
    
    fifo_add(url, title);

    g_free(title);
    g_free(url);

    gtk_widget_destroy(dialog);
}
