#include <gtk/gtk.h>
#include "light.h"

#include "menu.h"
#include "ui_cb.h"

static GtkItemFactoryEntry light_menu[] = {
    { "/_Browser", NULL, NULL, 0, "<Branch>" },
    { "/Browser/_New Window", "<alt>N", menu_handler, 10, NULL },
    { "/Browser/_Open Local File", "<alt>O", menu_handler, 11, NULL },
    { "/Browser/_Close Window", "<alt>W", menu_handler, 12, NULL },
    { "/Browser/Sa_ve Page...", "<alt>S", menu_handler, 13, NULL },
    { "/Browser/_Print Page", NULL, menu_handler, 17, NULL },

    { "/_Page", NULL, NULL, 0, "<Branch>" },
    { "/Page/_Find", "<alt>F", menu_handler, 20, NULL },
/*    { "/Page/Page So_urce", "<alt>U", menu_handler, 21, NULL }, */
    { "/Page/Text _Size", NULL, NULL, 0, "<Branch>" },
    { "/Page/Text Size/_Normal", "<alt>backslash", menu_handler, 22, NULL },
    { "/Page/Text Size/sep1", NULL, NULL, 0, "<Separator>" },
#ifndef SWEDISH_KEYBOARD /* [ and ] are not available on a swedish kbd */
    { "/Page/Text Size/_Larger", "<alt>bracketright", menu_handler, 23, NULL },
    { "/Page/Text Size/_Smaller", "<alt>bracketleft", menu_handler, 24, NULL },
#else
    { "/Page/Text Size/_Larger", "<alt>equal", menu_handler, 23, NULL },
    { "/Page/Text Size/_Smaller", "<alt>minus", menu_handler, 24, NULL },
#endif
    { "/Page/sep1", NULL, NULL, 0, "<Separator>" },

    { "/Page/_Reload", "<alt>R", menu_handler, 25, NULL },
    { "/Page/Reload (_No Cache)", "<alt><shift>R", menu_handler, 26, NULL },
    { "/Page/sep2", NULL, NULL, 0, "<Separator>" },
#ifndef ENGLISH_IS_NO1 
    { "/Page/_Character Set", NULL, NULL, 0, "<Branch>" },
#endif
    { "/Page/_Autodetect", NULL, NULL, 0, "<Branch>" },
    { "/Page/Autodetect/_Off", NULL, menu_handler, 40, "<RadioItem>" },
    { "/Page/Autodetect/_Chinese", NULL, menu_handler, 40, "/Page/Autodetect/Off" },
    { "/Page/Autodetect/East _Asian", NULL, menu_handler, 40, "/Page/Autodetect/Off" },
    { "/Page/Autodetect/_Japanese", NULL, menu_handler, 40, "/Page/Autodetect/Off" },
    { "/Page/Autodetect/_Korean", NULL, menu_handler, 40, "/Page/Autodetect/Off" },
    { "/Page/Autodetect/_Russian", NULL, menu_handler, 40, "/Page/Autodetect/Off" },
    { "/Page/Autodetect/_Simplified Chinese", NULL, menu_handler, 40, "/Page/Autodetect/Off" },
    { "/Page/Autodetect/_Traditional Chinese", NULL, menu_handler, 40, "/Page/Autodetect/Off" },
    { "/Page/Autodetect/_Ukrainian", NULL, menu_handler, 40, "/Page/Autodetect/Off" },
    { "/Page/sep3", NULL, NULL, 0, "<Separator>" },

    { "/Page/Pr_eferences...", NULL, menu_handler, 29, NULL },
    { "/Page/_Debug", "F1", menu_handler, 50, NULL },

    { "/_Utils", NULL, NULL, 0, "<Branch>" },
    { "/Utils/Open _Fifo", "<alt>I", menu_handler, 14, NULL },
    { "/Utils/Open _Search", "<alt>slash", menu_handler, 15, NULL },
    { "/Utils/sep1", NULL, NULL, 0, "<Separator>" },
    { "/Utils/_Go to Address entry", "<alt>D", menu_handler, 16, NULL },
    { "/Utils/_URL Entry to Fifo", "<alt>period", menu_handler, 33, NULL },

    { "/Utils/_View Toggle", NULL, NULL, 0, "<Branch>" },
    { "/Utils/View Toggle/_Menubar", "<Ctl>7", menu_handler, LIGHT_MENU, NULL },
    { "/Utils/View Toggle/_Toolbar", "<Ctl>8", menu_handler, LIGHT_TOOLBAR, NULL },
    { "/Utils/View Toggle/_Statusbar", "<Ctl>9", menu_handler, LIGHT_STATUSBAR, NULL },
    
    { "/_Help", NULL, NULL, 0, "<LastBranch>" },
    { "/Help/_Manual", NULL, menu_handler, 30, NULL },
    { "/Help/_About", NULL, menu_handler, 31, NULL },
};

GtkWidget *light_menubar_new(GtkAccelGroup *accel, Browser *browser)
{
    GtkItemFactory *fact;
    int count = sizeof(light_menu) / sizeof(light_menu[0]);

    fact = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel);
    gtk_item_factory_create_items(fact, count, light_menu, browser);
    browser->fact = fact;

    return gtk_item_factory_get_widget(fact, "<main>");
}

/**
 * the not-really-needed force charset menu.
 * this code doesn't have to be used really, since moz autodetect works
 * pretty damn good
 */
#ifndef ENGLISH_IS_NO1
extern char **charsets;

static MenuEntry charset_menu[] = {
    { "armscii-8",		"/Armenian/ARMSCII-8" },

    { "ISO-8859-4",		"/Baltic/ISO-8859-4" },
    { "ISO-8859-13",		"/Baltic/ISO-8859-13" },
    { "windows-1257",		"/Baltic/Windows-1256" },

    { "ISO-8859-14",		"/Celtic/ISO-8859-14" },

    { "IBM852",			"/Central European/IBM852" },
    { "ISO-8859-2",		"/Central European/ISO-8859-2" },
    { "x-mac-ce",		"/Central European/MacCE" },
    { "windows-1250",		"/Central European/Windows-1250" },

    { "x-gbk",			"/Chinese/GBK" },
    { "HZ-GB-2312",		"/Chinese/HZ-GB-2312" },
    { "GB2312",			"/Chinese/Simplified (GB2312)" },
    { "Big5",			"/Chinese/Traditional (BIG5)" },
    { "x-euc-tw",		"/Chinese/Traditional (EUC-TW)" },
    
    { "x-mac-croatian",		"/Croatian/MacCroatian" },

    { "ISO-8859-5",		"/Cyrillic/ISO-8859-5" },
    { "IBM855",			"/Cyrillic/IBM855" },
    { "ISO-IR-111",		"/Cyrillic/ISO-IR-111" },
    { "windows-1251",		"/Cyrillic/Windows-1251" },
    { "IBM866",			"/Cyrillic/Russian (IBM866)" },
    { "KOI8-R",			"/Cyrillic/Russian (KOI8-R)" },
    { "KOI8-U",			"/Cyrillic/Ukrainian (KOI8-U)" },
    { "x-mac-cyrillic",		"/Cyrillic/MacCyrillic" },
    { "x-mac-ukrainian",	"/Cyrillic/MacUkrainian" },

    { "ISO-8859-7",		"/Greek/ISO-8859-7" },
    { "windows-1253",		"/Greek/Windows-1253" },
    { "x-mac-greek",		"/Greek/MacGreek" },

    { "ISO-8859-8",		"/Hebrew/ISO-8859-8" },
    { "ISO-8859-8-I",		"/Hebrew/ISO-8859-8-I" },
    { "ISO-8859-8-E",		"/Hebrew/ISO-8859-8-E" },
    
    { "x-mac-icelandic",	"/Icelandic/MacIcelandic" },

    { "Shift_JIS",		"/Japanese/Shift-JIS" },
    { "EUC-JP",			"/Japanese/EUC-JP" },
    { "ISO-2022-JP",		"/Japanese/ISO-2022-JP" },

    { "EUC-KR",			"/Korean/EUC-KR" },

    { "ISO-8859-10",		"/Nordic/ISO-8859-10" },

    { "x-mac-romanian",		"/Romanian/MacRomanian" },

    { "ISO-8859-3",		"/South European/ISO-8859-3" },

    { "TIS-620",		"/Thai/TIS-620" },

    { "IBM857",			"/Turkish/IBM857" },
    { "ISO-8859-9",		"/Turkish/ISO-8859-9" },
    { "windows-1254",		"/Turkish/Windows-1254" },
    { "x-mac-turkish",		"/Turkish/MacTurkish" },

    { "ISO-8859-6",		"/Unknown/ISO-8859-6" },
    { "ISO-8859-6-I",		"/Unknown/ISO-8859-6-I" },
    { "ISO-8859-6-E",		"/Unknown/ISO-8859-6-E" },
    { "windows-936",		"/Unknown/windows-936" },
    { "IBM862",			"/Unknown/IBM862" },
    { "IBM864",			"/Unknown/IBM864" },
    { "IBM864i",		"/Unknown/IBM864i" },
    { "x-u-escaped",		"/Unknown/x-u-escaped" },
    { "windows-1255",		"/Unknown/Windows-1255" },
    { "windows-1256",		"/Unknown/Windows-1256" },
    { "T.61-8bit",		"/Unknown/T.61-8bit" },
    { "x-user-defined",		"/Unknown/User-Defined" },

    { "UTF-7",			"/Unicode/UTF-7" },
    { "UTF-8",			"/Unicode/UTF-8" },
    { "UTF-16BE",		"/Unicode/UTF-16BE" },
    { "UTF-16LE",		"/Unicode/UTF-16LE" },
    { "UTF-32BE",		"/Unicode/UTF-32BE" },
    { "UTF-32LE",		"/Unicode/UTF-32LE" },
    { "x-imap4-modified-utf7",	"/Unicode/Modified UTF-7" },

    { "windows-1258",		"/Vietnamese/Windows-1258" },
    { "x-viet-tcvn5712",	"/Vietnamese/TCVN5712" },
    { "VISCII",			"/Vietnamese/VISCII" },
    { "x-viet-vps",		"/Vietnamese/VPS" },

    { "ISO-8859-1",		"/Western/Standard (ISO-8859-1)" },
    { "us-ascii",		"/Western/Standard (us-ascii)" },
    { "windows-1252",		"/Western/Windows-1252" },
    { "ISO-8859-15",		"/Western/ISO-8859-15" },
    { "IBM850",			"/Western/IBM850" },
    { "x-mac-roman",		"/Western/MacRoman" },

    { NULL, NULL }
};

void fill_charset_menu(Browser *browser)
{
    GtkItemFactory *fact;
    GtkItemFactoryEntry entry;
    char *cpath = "/Page/Character Set";
    int i, j;

    fact = browser->fact;

    for (i = 0; charset_menu[i].name != NULL; i++) {
	for (j = 0; charsets[j] != NULL; j++) {
	    if (!g_strcasecmp(charsets[j], charset_menu[i].name)) {
		entry.path =
		    g_strconcat(cpath, charset_menu[i].path, NULL);
		entry.accelerator = NULL;
		entry.callback = menu_handler;
		entry.callback_action = 60 + j;
		entry.item_type = NULL;
		gtk_item_factory_create_item(fact, &entry, browser, 1);
		g_free(entry.path);
		break;
	    }
	}
    }
}
#endif
