#include <gtk/gtk.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "light.h"

#include "moz_pref.h"
#include "pref.h"

extern Config config;

void light_set_preferences(int first)
{
    /* set default preferences */
    if (first) {
	g_print
	    ("setting up default preferences.  check out ~/.light/prefs.js\n"
	     "and edit as necessary\n");

	/* this preference is broken - enable it when it actually works
	moz_pref_set_bool("nglayout.widget.gfxscrollbars", FALSE); */

	moz_pref_set_char(LT_USERAGENT, "Light 1.x");
	moz_pref_set_char(LT_HOMEPAGE, "about:blank");

	/* language stuff */
	moz_pref_set_char(LT_FONTDEFAULT, "sans-serif");
	moz_pref_set_char("font.name.sans-serif.x-western",
			  "adobe-helvetica-iso8859-1");
	moz_pref_set_char("font.name.serif.x-western",
			  "adobe-times-iso8859-1");
	moz_pref_set_int("font.size.variable.x-western", 14);

	/* setup cache */
	moz_pref_set_bool("browser.cache.enable", TRUE);
	moz_pref_set_bool("browser.cache.disk.enable", TRUE);
	moz_pref_set_int("browser.cache.disk_cache_size", 4096);
	moz_pref_set_bool("browser.cache.memory.enable", TRUE);
	moz_pref_set_int("browser.cache.memory_cache_size", 1024);

	/* setup network stuff */
	moz_pref_set_int("network.http.max-connections", 8);
	moz_pref_set_char("network.http.accept-encoding",
			  "gzip,deflate,compress,identity");
	moz_pref_set_char("network.search.url",
			  "http://www.google.com/search?q=");

	/* sekurity */
	moz_pref_set_bool(LT_JAVA, FALSE);
	moz_pref_set_bool(LT_JSCRIPT, TRUE);

	/* light specific prefs */
	moz_pref_set_char(LT_DOWNLOADER, "");
	moz_pref_set_int(LT_PIXMAPS, 2);
	moz_pref_set_int(LT_WIN_WIDTH, 800);
	moz_pref_set_int(LT_WIN_HEIGHT, 600);
	moz_pref_set_int(LT_FIFO_HEIGHT, 300);

	/* save prefs */
	moz_pref_save();
    }

    /* read in only the prefs requires for operation.  No sense reading in
     * mozilla-preferences if we don' display or use them! */
    
    /* language setting */
    moz_pref_get_char(LT_DETECTOR, &config.detector);

    /* get light-specific prefs */
    moz_pref_get_char(LT_DOWNLOADER, &config.handler);
    if (config.handler == NULL)
	config.handler = "";

    /* get custom search URL */
    moz_pref_get_char("network.search.url", &config.searchurl);
    if (config.searchurl == NULL)
	config.searchurl = "http://www.google.com/search?q=";

    /* window look */
    moz_pref_get_int(LT_PIXMAPS, &config.pixmaps);
    moz_pref_get_int(LT_WIN_WIDTH, &config.width);
    moz_pref_get_int(LT_WIN_HEIGHT, &config.height);
    moz_pref_get_int(LT_FIFO_HEIGHT, &config.fifoheight);

    /* check sanity and set "default" values so that they appear in prefs.js */
    if (config.width == 0) {
	config.width = 800;
	moz_pref_set_int(LT_WIN_WIDTH, 800);
    }
    if (config.height == 0) {
	config.height = 600;
	moz_pref_set_int(LT_WIN_HEIGHT, 600);
    }
    if (config.fifoheight == 0) {
	config.fifoheight = 300;
	moz_pref_set_int(LT_FIFO_HEIGHT, 300);
    }
}
