#include <gtk/gtk.h>
#include <gtkmozembed.h>
#include <gdk/gdkkeysyms.h>
#include <string.h>
#include "light.h"

#include "context.h"
#include "fifo.h"
#include "moz_main.h"
#include "moz_pref.h"
#include "ui.h"
#include "ui_cb.h"
#include "utils.h"

extern Config config;
extern GHashTable *detect;
#ifndef ENGLISH_IS_NO1
extern char **charsets;
#endif

/* global search marker */
static int found;

/* local stuff */
static void file_open_ok_cb(GtkWidget *button, GtkWidget *dialog);
static int file_open_dialog(GtkWidget *item, Browser *browser);

void toolbar_resize_cb(GtkWidget *widget, GtkAllocation *alloc, gpointer data)
{
    int height;
    int btnwidth;
    Browser *browser = (Browser *) data;
    static int ignore = 0;

    /* this is kind of a hack. we ignore toolbar resize event following
     * entry resize (or else we get in a endless loop) */
    if (ignore) {
	ignore = 0;
	return;
    }

    /* here, we calculate size of toobar buttons + space */
    height = GTK_WIDGET(browser->entry)->allocation.height;
    btnwidth = GTK_WIDGET(browser->btn.back)->allocation.width;

    /* 5 toolbar buttons + space */
    btnwidth += UI_SPACING * 2;
    btnwidth *= 5;
    /* minus 2 sets of space ("grab") */
    btnwidth -= UI_SPACING * 2;
    
    gtk_widget_set_usize(browser->entry, alloc->width - btnwidth, height);
    /* ignore the toolbar resize event that's going to follow */
    ignore = 1;
}

void dialog_keypress_cb(GtkWidget *widget, GdkEventKey *key, GtkWidget *window)
{
    /* GTK really sucks.  This is what we have to do to get normal
     * user accepted behavior of pressing "esc" key to get rid of a dialog
     * box */
    if (key->keyval == GDK_Escape) {
	gtk_widget_destroy(window);
	found = FALSE;
    }
}

void menu_handler(gpointer data, int action, GtkWidget * item)
{
    /* get our browser object since we'll need it */
    Browser *browser = (Browser *) data;
    char *uri = NULL;
    unsigned int chrome;

    chrome = gtk_moz_embed_get_chrome_mask(GTK_MOZ_EMBED(browser->embed));

    switch (action) {
    case 0:			/* null menu */
	break;
    case LIGHT_MENU:			/* show/hide menubar */
	if (GTK_WIDGET_VISIBLE(browser->menubar))
	    gtk_widget_hide(browser->menubar);
	else
	    gtk_widget_show_all(browser->menubar);
	chrome ^= GTK_MOZ_EMBED_FLAG_MENUBARON;
	gtk_moz_embed_set_chrome_mask(GTK_MOZ_EMBED(browser->embed), chrome);
	break;
    case LIGHT_TOOLBAR:		/* show/hide toolbar */
	if (GTK_WIDGET_VISIBLE(browser->toolbar))
	    gtk_widget_hide(browser->toolbar);
	else
	    gtk_widget_show_all(browser->toolbar);
	chrome ^= GTK_MOZ_EMBED_FLAG_TOOLBARON;
	gtk_moz_embed_set_chrome_mask(GTK_MOZ_EMBED(browser->embed), chrome);
	break;
    case LIGHT_STATUSBAR:		/* show/hide statusbar */
	if (GTK_WIDGET_VISIBLE(browser->statbar))
	    gtk_widget_hide(browser->statbar);
	else
	    gtk_widget_show_all(browser->statbar);
	chrome ^= GTK_MOZ_EMBED_FLAG_STATUSBARON;
	gtk_moz_embed_set_chrome_mask(GTK_MOZ_EMBED(browser->embed), chrome);
	break;
    case 10:			/* new window */
	light_window_new("about:blank");
	break;
    case 11:			/* open local file */
	file_open_dialog(item, browser);
	break;
    case 12:
	/* close_window();
	 * I suppose this will do all the proper reference counting */
	gtk_widget_destroy(browser->menubar);
	gtk_widget_destroy(browser->toolbar);
	gtk_widget_destroy(browser->window);
	break;
    case 13:			/* save as from File menu */
	uri = moz_get_main_document_url(browser);
	if (uri) {
	    char *final;
	    /* top-level site returns something like "http://localhost/"
	     * so, we append index.html at the end of it */
	    if (uri[strlen(uri) - 1] == '/')
		final = g_strconcat(uri, "index.html", NULL);
	    else
		final = g_strdup(uri);
	    g_free(uri);
	    light_save_as(browser, final);
	    g_free(final);
	}
	break;
    case 14:			/* open fifo popup */
	{
	    int x, y, width;
	    GtkWidget *window;

	    if (!fifo_size())
		break;

	    /* get position of main window */
	    gdk_window_get_position(browser->window->window,
		    &x, &y);

	    /* add up relative position of fifo button */
	    x += browser->btn.fifo->allocation.x;
	    y += browser->btn.fifo->allocation.y;

	    /* add up 1/2 of button size */
	    x += (browser->btn.fifo->allocation.width / 2);
	    y += (browser->btn.fifo->allocation.height / 2);

	    window = light_fifo_new(browser);
	    width = window->allocation.width;
	    gtk_window_reposition(GTK_WINDOW(window), x - width, y);
	    gtk_grab_add(window);
	}
	break;
    case 15:
	light_searchbox_new(browser);
	break;
    case 16:			/* get into URL entry box and select it */
	gtk_widget_grab_focus(browser->entry);
	gtk_editable_set_position(GTK_EDITABLE(browser->entry), 0);
	gtk_editable_select_region(GTK_EDITABLE(browser->entry), 0, -1);
	break;
    case 17:			/* print */
	moz_print(browser, NULL, "/tmp/mozilla.ps");
	break;
    case 20:			/* find dialog */
	light_find(browser);
	break;
    case 21:			/* view page source */
	g_print("page source - probably will not be implemented, I never use it\n");
	break;
    case 22:			/* normal size text */
	browser->size = 100;
	moz_set_size(browser, 1.0);
	break;
    case 23:
	browser->size += 10;	/* bigger text */
	moz_set_size(browser, browser->size / 100.0);
	break;
    case 24:
	browser->size -= 10;	/* smaller text */
	moz_set_size(browser, browser->size / 100.0);
	break;
    case 25:			/* reload standard */
	gtk_moz_embed_reload(GTK_MOZ_EMBED(browser->embed),
			     GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
	break;
    case 26:			/* reload no cache */
	gtk_moz_embed_reload(GTK_MOZ_EMBED(browser->embed),
			     GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE);
	break;
    case 29:			/* preferences "menu" */
	update_message(browser,
		"Preferences: Close all Light windows, and vi ~/.light/prefs.js");
	break;
    case 30:			/* manual menu */
	gtk_moz_embed_load_url(GTK_MOZ_EMBED(browser->embed), "about:manual");
	break;
    case 31:			/* about menu */
	gtk_moz_embed_load_url(GTK_MOZ_EMBED(browser->embed), "about:light");
	break;
    case 33:
	{
	    GdkEvent *event;
	    event = g_new0(GdkEvent, 1);
	    event->button.button = 3;
	    url_rightclick_cb(browser->entry, event, browser);
	    g_free(event);
	}
	break;
    case 40:			/* autodetect charset menu */
	{
	    char *value;
	    char *key;

	    if (browser->init)	/* break while making new window */
		break;

	    /* why the FUCK does GTK call activation for the item
	     * being unchecked? IDIOTS */
	    if (!GTK_CHECK_MENU_ITEM(item)->active)
		break;

	    key = gtk_item_factory_path_from_widget(item);
	    value = g_hash_table_lookup(detect, key);

	    if (value && strlen(value) > 0) {
		moz_pref_set_char(LT_DETECTOR, value);
		config.detector = value;
#ifndef ENGLISH_IS_NO1
		/* override "forced" charset */
		moz_pref_set_char(LT_DEFCHARSET, NULL);
#endif
	    } else {
		/* clear detector value */
		moz_pref_clear(LT_DETECTOR);
	    }
	    /* reload the page */
	    gtk_moz_embed_reload(GTK_MOZ_EMBED(browser->embed),
				 GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
	}
	break;
    case 50:			/* debug handler */
	g_print("hahahahha! %p\n", browser);
	break;
    default:			/* charset menus - 60+ */
#ifndef ENGLISH_IS_NO1
	{
	    GtkWidget *menu;

	    /* turn off autodetect menu item */
	    config.detector = "";
	    moz_pref_clear(LT_DETECTOR);
	    moz_set_charset(browser, charsets[action - 60]);
	    moz_pref_set_char(LT_DEFCHARSET, charsets[action - 60]);
	    gtk_moz_embed_reload(GTK_MOZ_EMBED(browser->embed),
				 GTK_MOZ_EMBED_FLAG_RELOADNORMAL);

	    /* turn off autodetect menu */
	    menu = gtk_item_factory_get_item(browser->fact,
		    "/Page/Autodetect/Off");
	    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu),
		    TRUE);
	}
#endif
	break;
    }
}

/* This happens when <Enter> is pressed in the URL entry box */
void url_activate_cb(GtkEditable *widget, Browser *browser)
{
    char *text = gtk_editable_get_chars(widget, 0, -1);
    gtk_moz_embed_load_url(GTK_MOZ_EMBED(browser->embed), text);
    g_free(text);
}

/* Called when URL is clicked */
int url_click_cb(GtkWidget *widget, GdkEventButton *event, Browser *browser)
{
    if (event->type == GDK_2BUTTON_PRESS && event->button == 1) {
	gtk_editable_set_position(GTK_EDITABLE(widget), -1);
	gtk_entry_select_region(GTK_ENTRY(widget), 0, -1);
	gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
		"button-press-event");
	return TRUE;
    }
    return FALSE;
}

/* called when right-clicking on the URL entry box */
void url_rightclick_cb(GtkWidget *widget, GdkEvent *event, Browser *browser)
{
    if (event->button.button == 3) {
	char *contents = NULL;
	contents = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	if (contents && (strlen(contents) > 0))
	    light_add_fifo(browser, contents, NULL);
    }
}

/* when ESC is pressed inside the URL entry box, deselect its contents and
 * go back to the browser area */
void url_keypress_cb(GtkWidget *widget, GdkEventKey *key, Browser *browser)
{
    if (key->keyval == GDK_Escape) {
	gtk_editable_select_region(GTK_EDITABLE(browser->entry),
		0, 0);
	gtk_widget_grab_focus(browser->embed);
    }
}

void mainwindow_destroy_cb(GtkWidget *widget, Browser *browser)
{
    /* need to do that or else the throbber loop never stops and keeps
     * writing to a non-existent GtkLabel */
    if (browser->load.throbber)
	gtk_timeout_remove(browser->load.throbber);
}

/* find dialog callback */
void find_dialog_close_cb(GtkWidget *widget, GtkWidget *window)
{
    found = FALSE;
    gtk_widget_destroy(window);
}

void find_dialog_clear_cb(GtkWidget *widget, GtkWidget *window)
{
    GtkWidget *entry;

    found = FALSE;

    entry = gtk_object_get_data(GTK_OBJECT(window), "entry");
    gtk_editable_delete_text(GTK_EDITABLE(entry), 0,
			     GTK_ENTRY(entry)->text_length);
    gtk_widget_grab_focus(entry);
}

void find_dialog_find_cb(GtkWidget *widget, GtkWidget *window)
{
    GtkWidget *entry;
    GtkWidget *sens;
    GtkWidget *back;
    Browser *browser;

    browser = gtk_object_get_data(GTK_OBJECT(window), "browser");

    entry = gtk_object_get_data(GTK_OBJECT(window), "entry");
    sens = gtk_object_get_data(GTK_OBJECT(window), "sens");
    back = gtk_object_get_data(GTK_OBJECT(window), "back");

    if (GTK_ENTRY(entry)->text_length == 0)
	return;

    /* remember our last find value */
    if (browser->find)
	g_free(browser->find);
    browser->find = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);

    moz_find(browser,
	      gtk_entry_get_text(GTK_ENTRY(entry)),
	      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sens)),
	      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(back)),
	      &found);

    if (!found) {
	GtkWidget *dialog;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *button;

	dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), "Find");
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
	gtk_container_set_border_width(GTK_CONTAINER
				       (GTK_DIALOG(dialog)->action_area),
				       UI_SPACING);
	gtk_signal_connect(GTK_OBJECT(dialog), "key_press_event",
			   GTK_SIGNAL_FUNC(dialog_keypress_cb), dialog);

	hbox = gtk_hbox_new(FALSE, UI_SPACING);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), UI_SPACING);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, TRUE,
			   TRUE, 0);

	label = gtk_label_new("No more matches or text not found");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);

	button = gtk_button_new_with_label("OK");
	GTK_WIDGET_SET_FLAGS(button, GTK_HAS_DEFAULT);
	gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
				  GTK_SIGNAL_FUNC(gtk_widget_destroy),
				  (gpointer) dialog);

	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
			   button, TRUE, TRUE, 0);

	gtk_widget_show_all(dialog);
	gtk_widget_grab_default(button);
	gtk_widget_grab_focus(button);
    }
    gtk_widget_grab_focus(entry);
}

/* search box callback */
void search_activate_cb(GtkEditable *widget, Browser *browser)
{
    int type;
    char *text;
    char *request;
    GtkWidget *window;

    type = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(widget),
		"search_type"));
    window = gtk_object_get_data(GTK_OBJECT(widget), "window");
    request = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
    text = escape_string(request, 0);
    /* reuse it later */
    g_free(request);

    switch (type) {
	case 10:		/* google search */
	    request = g_strconcat("http://www.google.com/search?q=",
		    text, NULL);
	    break;
	case 20:		/* freshmeat search */
	    request = g_strconcat("http://freshmeat.net/search?q=",
		    text, NULL);
	    break;
	case 30:		/* custom search */
	    request = g_strdup_printf(config.searchurl, text);
	    break;
    }

    gtk_moz_embed_load_url(GTK_MOZ_EMBED(browser->embed), request);
    g_free(request);
    g_free(text);

    gtk_widget_destroy(window);
}

/* 4 toolbar buttons */
void back_cb(GtkButton *button, Browser *browser)
{
    gtk_moz_embed_go_back(GTK_MOZ_EMBED(browser->embed));
}

void stop_cb(GtkButton *button, Browser *browser)
{
    gtk_moz_embed_stop_load(GTK_MOZ_EMBED(browser->embed));
}

void forward_cb(GtkButton *button, Browser *browser)
{
    gtk_moz_embed_go_forward(GTK_MOZ_EMBED(browser->embed));
}

void reload_cb(GtkButton *button, Browser *browser)
{
    gtk_moz_embed_reload(GTK_MOZ_EMBED(browser->embed),
			 GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
}

void fifo_pressed_cb(GtkWidget *widget, GdkEventButton *event, Browser *browser)
{
    GtkWidget *window;
    /* for positioning */
    int width, x, y;

    if (event->button != 1 || !fifo_size())
	return;

    window = light_fifo_new(browser);

    /* move the window so that upper-right corner is where the mouse is */
    width = window->allocation.width;
    x = (int)event->x_root;
    y = (int)event->y_root;
    gtk_window_reposition(GTK_WINDOW(window), x - width, y);

    gtk_grab_add(window);
}

static void file_open_ok_cb(GtkWidget *button, GtkWidget *dialog)
{
    char *filename;
    Browser *browser;
    char *final;

    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(dialog));
    browser = gtk_object_get_data(GTK_OBJECT(dialog), "browser");

    if (!filename || !strcmp(filename, ""))
	return;

    final = g_strdup_printf("file://%s", filename);
    gtk_moz_embed_load_url(GTK_MOZ_EMBED(browser->embed), final);
    g_free(final);
    
    gtk_widget_destroy(dialog);
}

static int file_open_dialog(GtkWidget *item, Browser *browser)
{
    GtkWidget *dialog;

    dialog = gtk_file_selection_new("Open File...");
    gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
    gtk_object_set_data(GTK_OBJECT(dialog), "browser", browser);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(dialog)->ok_button),
		       "clicked", GTK_SIGNAL_FUNC(file_open_ok_cb),
		       dialog);
    gtk_signal_connect_object(GTK_OBJECT
			      (GTK_FILE_SELECTION(dialog)->cancel_button),
			      "clicked",
			      GTK_SIGNAL_FUNC(gtk_widget_destroy),
			      (gpointer) dialog);
    gtk_signal_connect(GTK_OBJECT(dialog), "key_press_event",
		       GTK_SIGNAL_FUNC(dialog_keypress_cb), dialog);
    gtk_widget_show_all(dialog);

    return TRUE;
}
